@@ -6,23 +6,28 @@ Also includes a function to aggregate common key term variants of the same idea.
 """
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from collections import Counter, defaultdict
-from decimal import Decimal
+import collections
 import itertools
 import logging
 import math
-from operator import itemgetter
+import operator
+from decimal import Decimal
 
-from cytoolz import itertoolz
 import networkx as nx
 import numpy as np
+from cytoolz import itertoolz
 
-from textacy import extract, vsm
-from textacy.network import terms_to_semantic_network
-from textacy.similarity import token_sort_ratio
+from . import compat
+from . import extract
+from . import network
+from . import similarity
+from . import vsm
+
+LOGGER = logging.getLogger(__name__)
 
 
-def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500, n_keyterms=10, idf=None):
+def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
+           n_keyterms=10, idf=None):
     """
     Extract key terms from a document using the [SGRank]_ algorithm.
 
@@ -45,11 +50,11 @@ def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
             ``int(round(len(doc) * n_keyterms))``
         idf (dict): Mapping of ``normalize(term)`` to inverse document frequency
             for re-weighting of unigrams (n-grams with n > 1 have df assumed = 1).
-            NOTE: Results are typically better with idf information.
+            Results are typically better with idf information.
 
     Returns:
         List[Tuple[str, float]]: sorted list of top ``n_keyterms`` key terms and
-            their corresponding SGRank scores
+        their corresponding SGRank scores
 
     Raises:
         ValueError: If ``n_keyterms`` is a float but not in (0.0, 1.0] or
@@ -70,7 +75,7 @@ def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
     window_width = min(n_toks, window_width)
     min_term_freq = min(n_toks // 1000, 4)
     if isinstance(ngrams, int):
-            ngrams = (ngrams,)
+        ngrams = (ngrams,)
 
     # build full list of candidate terms
     # if inverse doc freqs available, include nouns, adjectives, and verbs;
@@ -87,7 +92,7 @@ def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
     if normalize == 'lemma':
         terms = [(term.lemma_, term.start, len(term)) for term in terms]
     elif normalize == 'lower':
-        terms = [(term.orth_.lower(), term.start, len(term)) for term in terms]
+        terms = [(term.lower_, term.start, len(term)) for term in terms]
     elif not normalize:
         terms = [(term.text, term.start, len(term)) for term in terms]
     else:
@@ -95,14 +100,14 @@ def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
 
     # pre-filter terms to the top N ranked by TF or modified TF*IDF
     n_prefilter_kts = max(3 * n_keyterms, 100)
-    term_text_counts = Counter(term[0] for term in terms)
+    term_text_counts = collections.Counter(term[0] for term in terms)
     if idf:
         mod_tfidfs = {
             term: count * idf.get(term, 1) if ' ' not in term else count
             for term, count in term_text_counts.items()}
         terms_set = {
             term for term, _
-            in sorted(mod_tfidfs.items(), key=itemgetter(1), reverse=True)[:n_prefilter_kts]}
+            in sorted(mod_tfidfs.items(), key=operator.itemgetter(1), reverse=True)[:n_prefilter_kts]}
     else:
         terms_set = {term for term, _ in term_text_counts.most_common(n_prefilter_kts)}
     terms = [term for term in terms if term[0] in terms_set]
@@ -132,12 +137,12 @@ def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
     # filter terms to only those with positive weights
     terms = [term for term in terms if term_weights[term[0]] > 0]
 
-    n_coocs = defaultdict(lambda: defaultdict(int))
-    sum_logdists = defaultdict(lambda: defaultdict(float))
+    n_coocs = collections.defaultdict(lambda: collections.defaultdict(int))
+    sum_logdists = collections.defaultdict(lambda: collections.defaultdict(float))
 
     # iterate over windows
     log_ = math.log  # localize this, for performance
-    for start_ind in range(n_toks):
+    for start_ind in compat.range_(n_toks):
         end_ind = start_ind + window_width
         window_terms = (term for term in terms
                         if start_ind <= term[1] <= end_ind)
@@ -149,7 +154,7 @@ def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
             break
 
     # compute edge weights between co-occurring terms (nodes)
-    edge_weights = defaultdict(lambda: defaultdict(float))
+    edge_weights = collections.defaultdict(lambda: collections.defaultdict(float))
     for t1, t2s in sum_logdists.items():
         for t2 in t2s:
             edge_weights[t1][t2] = ((1.0 + sum_logdists[t1][t2]) / n_coocs[t1][t2]) * term_weights[t1] * term_weights[t2]
@@ -165,7 +170,7 @@ def sgrank(doc, ngrams=(1, 2, 3, 4, 5, 6), normalize='lemma', window_width=1500,
     graph.add_edges_from(norm_edge_weights)
     term_ranks = nx.pagerank_scipy(graph)
 
-    return sorted(term_ranks.items(), key=itemgetter(1, 0), reverse=True)[:n_keyterms]
+    return sorted(term_ranks.items(), key=operator.itemgetter(1, 0), reverse=True)[:n_keyterms]
 
 
 def textrank(doc, normalize='lemma', n_keyterms=10):
@@ -259,7 +264,7 @@ def key_terms_from_semantic_network(doc, normalize='lemma',
 
     Returns:
         List[Tuple[str, float]]: sorted list of top ``n_keyterms`` key terms and
-            their corresponding ranking scores
+        their corresponding ranking scores
 
     Raises:
         ValueError: if ``n_keyterms`` is a float but not in (0.0, 1.0]
@@ -291,7 +296,7 @@ def key_terms_from_semantic_network(doc, normalize='lemma',
     # and may well happen with ``normalize`` as a callable
     # an empty string should never be considered a keyterm
     good_word_list = [word for word in good_word_list if word]
-    graph = terms_to_semantic_network(
+    graph = network.terms_to_semantic_network(
         good_word_list, window_width=window_width, edge_weighting=edge_weighting)
 
     # rank nodes by algorithm, and sort in descending order
@@ -307,11 +312,11 @@ def key_terms_from_semantic_network(doc, normalize='lemma',
     # bail out here if all we wanted was key *words* and not *terms*
     if join_key_words is False:
         return [(word, score) for word, score in
-                sorted(word_ranks.items(), key=itemgetter(1), reverse=True)[:n_keyterms]]
+                sorted(word_ranks.items(), key=operator.itemgetter(1), reverse=True)[:n_keyterms]]
 
     top_n = int(0.25 * len(word_ranks))
     top_word_ranks = {word: rank for word, rank in
-                      sorted(word_ranks.items(), key=itemgetter(1), reverse=True)[:top_n]}
+                      sorted(word_ranks.items(), key=operator.itemgetter(1), reverse=True)[:top_n]}
 
     # join consecutive key words into key terms
     seen_joined_key_terms = set()
@@ -325,7 +330,7 @@ def key_terms_from_semantic_network(doc, normalize='lemma',
             seen_joined_key_terms.add(term)
             joined_key_terms.append((term, sum(word_ranks[word] for word in words)))
 
-    return sorted(joined_key_terms, key=itemgetter(1, 0), reverse=True)[:n_keyterms]
+    return sorted(joined_key_terms, key=operator.itemgetter(1, 0), reverse=True)[:n_keyterms]
 
 
 def most_discriminating_terms(terms_lists, bool_array_grp1,
@@ -349,12 +354,13 @@ def most_discriminating_terms(terms_lists, bool_array_grp1,
 
     Returns:
         List[str]: top `top_n_terms` most discriminating terms for grp1-not-grp2
+
         List[str]: top `top_n_terms` most discriminating terms for grp2-not-grp1
 
     References:
         King, Gary, Patrick Lam, and Margaret Roberts. "Computer-Assisted Keyword
-            and Document Set Discovery from Unstructured Text." (2014).
-            http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.458.1445&rep=rep1&type=pdf
+        and Document Set Discovery from Unstructured Text." (2014).
+        http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.458.1445&rep=rep1&type=pdf
     """
     alpha_grp1 = 1
     alpha_grp2 = 1
@@ -363,20 +369,21 @@ def most_discriminating_terms(terms_lists, bool_array_grp1,
     bool_array_grp1 = np.array(bool_array_grp1)
     bool_array_grp2 = np.invert(bool_array_grp1)
 
-    dtm, id2term = vsm.doc_term_matrix(
-        terms_lists, weighting='tf', normalize=False,
-        sublinear_tf=False, smooth_idf=True,
-        min_df=3, max_df=0.95, min_ic=0.0, max_n_terms=max_n_terms)
+    vectorizer = vsm.Vectorizer(
+        tf_type='linear', norm=None, idf_type='smooth',
+        min_df=3, max_df=0.95, max_n_terms=max_n_terms)
+    dtm = vectorizer.fit_transform(terms_lists)
+    id2term = vectorizer.id_to_term
 
     # get doc freqs for all terms in grp1 documents
     dtm_grp1 = dtm[bool_array_grp1, :]
     n_docs_grp1 = dtm_grp1.shape[0]
-    doc_freqs_grp1 = vsm.get_doc_freqs(dtm_grp1, normalized=False)
+    doc_freqs_grp1 = vsm.get_doc_freqs(dtm_grp1)
 
     # get doc freqs for all terms in grp2 documents
     dtm_grp2 = dtm[bool_array_grp2, :]
     n_docs_grp2 = dtm_grp2.shape[0]
-    doc_freqs_grp2 = vsm.get_doc_freqs(dtm_grp2, normalized=False)
+    doc_freqs_grp2 = vsm.get_doc_freqs(dtm_grp2)
 
     # get terms that occur in a larger fraction of grp1 docs than grp2 docs
     term_ids_grp1 = np.where(doc_freqs_grp1 / n_docs_grp1 > doc_freqs_grp2 / n_docs_grp2)[0]
@@ -408,7 +415,7 @@ def most_discriminating_terms(terms_lists, bool_array_grp1,
         grp1_terms_likelihoods[id2term[term_id]] = term1 * term2
     top_grp1_terms = [term for term, likelihood
                       in sorted(grp1_terms_likelihoods.items(),
-                                key=itemgetter(1), reverse=True)[:top_n_terms]]
+                                key=operator.itemgetter(1), reverse=True)[:top_n_terms]]
 
     # get grp2 terms likelihoods, then sort for most discriminating grp2-not-grp1 terms
     grp2_terms_likelihoods = {}
@@ -418,7 +425,7 @@ def most_discriminating_terms(terms_lists, bool_array_grp1,
         grp2_terms_likelihoods[id2term[term_id]] = term1 * term2
     top_grp2_terms = [term for term, likelihood
                       in sorted(grp2_terms_likelihoods.items(),
-                                key=itemgetter(1), reverse=True)[:top_n_terms]]
+                                key=operator.itemgetter(1), reverse=True)[:top_n_terms]]
 
     return (top_grp1_terms, top_grp2_terms)
 
@@ -522,7 +529,7 @@ def aggregate_term_variants(terms,
             for other_term in sorted(terms.difference(seen_terms), key=len, reverse=True):
                 if len(other_term) < 13:
                     break
-                tsr = token_sort_ratio(term, other_term)
+                tsr = similarity.token_sort_ratio(term, other_term)
                 if tsr > 0.93:
                     variants.add(other_term)
                     seen_terms.add(other_term)
@@ -549,7 +556,7 @@ def rank_nodes_by_bestcoverage(graph, k, c=1, alpha=1.0):
 
     Returns:
         dict: top ``k`` nodes as ranked by bestcoverage algorithm; keys as node
-            identifiers, values as corresponding ranking scores
+        identifiers, values as corresponding ranking scores
 
     References:
         .. [BestCoverage] Küçüktunç, O., Saule, E., Kaya, K., & Çatalyürek, Ü. V.
@@ -560,13 +567,13 @@ def rank_nodes_by_bestcoverage(graph, k, c=1, alpha=1.0):
     """
     alpha = float(alpha)
 
-    nodes_list = graph.nodes()
+    nodes_list = list(dict(graph.nodes()))
 
     # ranks: array of PageRank values, summing up to 1
     ranks = nx.pagerank_scipy(graph, alpha=0.85, max_iter=100, tol=1e-08, weight='weight')
-    sorted_ranks = sorted(ranks.items(), key=itemgetter(1), reverse=True)
+    sorted_ranks = sorted(ranks.items(), key=operator.itemgetter(1), reverse=True)
 
-    avg_degree = sum(deg for _, deg in graph.degree_iter()) / len(nodes_list)
+    avg_degree = sum(deg for _, deg in graph.degree()) / len(nodes_list)
     # relaxation parameter, k' in the paper
     k_prime = int(k * avg_degree * c)
 
@@ -585,7 +592,7 @@ def rank_nodes_by_bestcoverage(graph, k, c=1, alpha=1.0):
         s = set(vertices)
         # s.update(vertices)
         # for each step
-        for _ in range(l):
+        for _ in compat.range_(l):
             # for each node
             next_vertices = []
             for vertex in vertices:
@@ -599,7 +606,7 @@ def rank_nodes_by_bestcoverage(graph, k, c=1, alpha=1.0):
     top_k_exp_vertices = get_l_step_expanded_set([item[0] for item in top_k_sorted_ranks], c)
 
     # compute initial exprel contribution
-    taken = defaultdict(bool)
+    taken = collections.defaultdict(bool)
     contrib = {}
     for vertex in nodes_list:
         # get l-step expanded set
@@ -610,11 +617,11 @@ def rank_nodes_by_bestcoverage(graph, k, c=1, alpha=1.0):
     sum_contrib = 0.0
     results = {}
     # greedily select to maximize exprel metric
-    for _ in range(k):
+    for _ in compat.range_(k):
         if not contrib:  # TODO: check that .items(): not needed
             break
         # find word with highest l-step expanded relevance score
-        max_word_score = sorted(contrib.items(), key=itemgetter(1), reverse=True)[0]
+        max_word_score = sorted(contrib.items(), key=operator.itemgetter(1), reverse=True)[0]
         sum_contrib += max_word_score[1]  # contrib[max_word[0]]
         results[max_word_score[0]] = max_word_score[1]
         # find its l-step expanded set
@@ -630,7 +637,8 @@ def rank_nodes_by_bestcoverage(graph, k, c=1, alpha=1.0):
                 try:
                     contrib[w] -= alpha * ranks[vertex]
                 except KeyError:
-                    logging.error('Word %s not in contrib dict! We\'re approximating...', w)
+                    LOGGER.error(
+                        'Word %s not in contrib dict! We\'re approximating...', w)
             taken[vertex] = True
         contrib[max_word_score[0]] = 0
 
@@ -661,7 +669,7 @@ def rank_nodes_by_divrank(graph, r=None, lambda_=0.5, alpha=0.5):
     """
     # check function arguments
     if len(graph) == 0:
-        logging.warning('``graph`` is empty!')
+        LOGGER.warning('``graph`` is empty!')
         return {}
 
     # create adjacency matrix, i.e.
@@ -703,10 +711,10 @@ def rank_nodes_by_divrank(graph, r=None, lambda_=0.5, alpha=0.5):
 
     # sort nodes by divrank score
     results = sorted(((i, score) for i, score in enumerate(pr.flatten().tolist())),
-                     key=itemgetter(1), reverse=True)
+                     key=operator.itemgetter(1), reverse=True)
 
     # replace node number by node value
-    nodes_list = graph.nodes()
+    nodes_list = list(dict(graph.nodes()))
     divranks = {nodes_list[result[0]]: result[1] for result in results}
 
     return divranks
